@@ -132,6 +132,13 @@ module NumRu
         elsif ! file.is_a?(GrADS_Gridded)
           raise ArgumentError, "1st arg must be a GrADS_Gridded or a file name"
         end
+        if( ! file.dset )
+          bfname = file.path.gsub(/\.ctl/,"")+".dat"
+          while( File.exist?(bfname) )
+            bfname = bfname + "_"
+          end
+          file.dset = bfname
+        end
         VArrayGrADS.write_control(file, gphys)
         VArrayGrADS.write_binary(file, gphys.data)
       end
@@ -151,7 +158,7 @@ module NumRu
         return var_names
       end
       def var_names_except_coordinates(file)
-	var_names(file)
+        var_names(file)
       end
 
     end
@@ -191,13 +198,13 @@ if $0 == __FILE__
    GPhys::NetCDF_IO.write(file2,temp_edy)
    file2.close
    file3 = NetCDF.create('tmp2.nc')
-   GPhys::NetCDF_IO.write(file2,temp_xmean)
+   GPhys::NetCDF_IO.write(file3,temp_xmean)
    file3.close
 
    puts "\n** test write (tmp.ctl) **"
-   file1 = GrADS_Gridded.create("tmp2.ctl")
+   file1 = GrADS_Gridded.create("tmp.ctl")
    file1.put_att("big_endian",true)
-   file1.put_att("dset","tmp2.dr")
+   file1.put_att("dset","tmp.dr")
    file1.put_att("title","test control file")
    file1.put_att("undef",-999.0)
    GPhys::GrADS_IO.write(file1,temp_edy)
