@@ -109,7 +109,8 @@ a NetCDF read/write helper by automatically interpreting conventions
     * grid_or_gphys (Grid or GPhys):
 
     RETURN VALUE
-    * nil
+    * a Grid, in which all VArrays in the original grid are replaced 
+      with the new ones in the file.
 
 ---each_along_dims_write(gphyses, files, *loopdims){...}  # a block is expected
 
@@ -303,7 +304,7 @@ module NumRu
       def is_a_NetCDF?(filename)
         file = nil
         begin
-          file = File.open(filename)
+          file = File.open(filename,"rb")
           str = file.read(3)
         ensure
           file.close if file
@@ -328,6 +329,7 @@ module NumRu
 	    bare_index[i]=true
 	    na = NArray.float(ncvar0.shape_current[i]).indgen!
 	    axpos = VArray.new( na )
+            axpos.name = "axis_#{i}"          # added by S. OTSUKA
 	  end
 	  cell_center, cell_bounds_name = convention::cell_center?( axpos )
 	  cell_bounds, cell_center_name = convention::cell_bounds?( axpos )
@@ -371,13 +373,14 @@ module NumRu
       end
 
       def write_grid(file, grid_or_gphys)
+	newaxes = Array.new
 	(0...(grid_or_gphys.rank)).each{|i|
 	  ax = grid_or_gphys.axis(i)
 	  dimname = ax.pos.name
 	  length = ax.pos.length
 	  isfx = 0
 	  altdimnames = Hash.new
-	  ax.flatten.each{ |va|
+	  newax = ax.collect{ |va|
 	    if va.length == length
 	      dimnames = [dimname]
 	    else
@@ -389,18 +392,22 @@ module NumRu
 	      end
 	    end
 	    if !(already=file.var(va.name))
-	      VArrayNetCDF.write(file, va, nil, dimnames )
+	      newva = VArrayNetCDF.write(file, va, nil, dimnames )
 	    else
               #< var with the same name exists --> check it >
               if va.shape_current != already.shape_current
 		raise "#{va.name} exisits but the shape is different" 
 	      #elsif va[0].val != already[0].val #(not allowed in the def mode)
 	      #	raise "#{va.name} exisits but the first element doesn't agree" 
+              else
+		newva = VArrayNetCDF.new(already)
 	      end
 	    end
+	    newva
 	  }
+	  newaxes.push( newax )
 	}
-	nil
+	Grid.new( *newaxes )
       end
 
       def def_var(file, name, ntype, dimensions, vary=nil)
@@ -654,11 +661,8 @@ end
 if $0 == __FILE__
    include NumRu
 
-   begin
-     file = NetCDF.open("../../testdata/T.jan.nc")
-   rescue
-     file = NetCDF.open("../../../testdata/T.jan.nc")
-   end
+   file = NetCDF.open("../../../testdata/T.jan.nc")
+
    temp = GPhys::NetCDF_IO.open(file,"T")
    p temp.name, temp.shape_current
    p temp.val.class
@@ -728,4 +732,18 @@ if $0 == __FILE__
     [s1.mean(1),s2.rename('T_x_mean'),s2.mean(0).rename('T_xy_mean')]
   }
   f.close
+
+  print "\n\n** PACKED DATA TREATMENT **\n\n"
+
+  file = NetCDF.open("../../../testdata/T.jan.packed.withmiss.nc")
+  temp = GPhys::NetCDF_IO.open(file,"T")
+  temp.att_names.each{|nm| p nm,temp.get_att(nm) if /(scale|offs)/ =~ nm}
+  p( mls=temp.copy.att_names )
+  p( (temp*10).att_names - mls )
+  p( temp[0,false].copy.att_names - mls )
+
+  print "\n\n** copying with write_grid **\n\n"
+  f=NetCDF.create('tmpE4.nc')
+  grid = GPhys::NetCDF_IO.write_grid(f,temp)
+  p grid,grid.axis(0).pos.val
 end
