@@ -33,9 +33,7 @@ through which the users can also access its binary data file(s).
 
    ARGUMENTS
    * ctlfilename (String): name of the control file to open
-   * mode (String): IO mode. "r" or "w+" (precisely speaking, you can
-     use "w", but it is interpreted as "w+". Thus, all files opened 
-     will be readable). CURRENTLY, ONLY "r" IS HANDLED.
+   * mode (String): IO mode. "r" (read only) or "w" (write only).
 
    REMARK
    * You can modify the object through instance methods even if mode=="r".
@@ -310,7 +308,7 @@ class GrADS_Gridded
         raise "share = true is not supported."
       end
       #if (File.exists?(ctlfilename))
-      if(!noclobber && File.exists?(ctlfilename))
+      if(noclobber && File.exists?(ctlfilename))
         print "#{ctlfilename} already exists.\n"
         print "overwrite #{ctlfilename} (y/n)? "
         ans = gets[0].chr
@@ -404,11 +402,12 @@ class GrADS_Gridded
 #  def def_dim(ary)
 
   def put_att(key,value)
-    if(key == "dset")
+    case key
+    when "dset"
       @dset = value
-    elsif(key == "title")
+    when "title"
       @title = value
-    elsif(key == "undef")
+    when "undef"
       @undef = value
     else
       if ! (value.is_a?(TrueClass) || value.is_a?(NilClass) )
@@ -454,6 +453,8 @@ class GrADS_Gridded
       att = @dset
     when("title")
       att = @title
+    when "undef"
+      att = @undef
     else
       raise "Invalid/unsupported option: "+key
     end
@@ -478,13 +479,20 @@ class GrADS_Gridded
   end
 
   def att_names
-    ary = ["dset","title"]
+    ary = ["dset","title","undef"]
     ary
   end
 
   def to_ctl
+    if( !@dimensions[3][:spec] ) 
+      start = generate_starttime(@dimensions[3][:startjd])
+      increment = generate_timeincrement(@dimensions[3][:increment],@dimensions[3][:increment_units])
+      @dimensions[3][:spec] = "#{start} #{increment}"
+    end
+    @title = "<no title>" if @title==""
+    @undef = -999.0 if @undef==nil
     return <<EOS
-DSET    #{@dset}
+DSET    #{if @dset[0]=="/" then @dset else "^"+@dset end}
 TITLE   #{@title}
 UNDEF   #{@undef}
 OPTIONS #{op=""; @options.each{|key,val| op += key+" " if(val)}; op}
@@ -493,7 +501,7 @@ YDEF    #{@dimensions[1][:len]} #{@dimensions[1][:flag]} #{@dimensions[1][:spec]
 ZDEF    #{@dimensions[2][:len]} #{@dimensions[2][:flag]} #{@dimensions[2][:spec]}
 TDEF    #{@dimensions[3][:len]} #{@dimensions[3][:flag]} #{@dimensions[3][:spec]}
 VARS    #{nvars}
-#{@variables.collect{|i| "  "+i[:name]+" "+i[:nlev].to_s+" "+i[:option].to_s+
+#{@variables.collect{|i| i[:name]+" "+i[:nlev].to_s+" "+i[:option].to_s+
        " "+i[:description]}.join("\n")}
 ENDVARS
 EOS
@@ -518,20 +526,22 @@ EOS
 
   def put(ary)
     ary = ary.to_na.hton if ( @options["big_endian"] )
-    if File.exist?(@dset)
-      print "overwrite "+@dset+" (y/n)? "
-      ans = gets[0].chr
-      if ans != "y"
-        print @dset+" remain unmodified. \n"
-        return
-      end
-    end
-    putfile = File.open(@dset,"w")
+#    if File.exist?(@dset)
+#      print "overwrite "+@dset+" (y/n)? "
+#      ans = gets[0].chr
+#      if ans != "y"
+#        print @dset+" remain unmodified. \n"
+#        return
+#      end
+#    end
+#    putfile = File.open(@dset,"w")
+    putfile = File.open(@dset,"wb")
     putfile << ary.to_s
     putfile.close
   end
 
-  def get(name, level, time)
+#  def get(name, level, time)
+  def get(name, level, time, lonlat=nil)
     if ( @options["template"] )
 #      time_a = get_dim(@dimensions[3]) # julian day(integer) + time of day
       time_a = get_dim(@dimensions[3])
@@ -567,19 +577,46 @@ EOS
       init_in_file = (time_a.ge time_i).count_false
       time_in_file = time - init_in_file
       start_byte = start_byte(name, level, time_in_file)
-#      @datafile = File.open(@dset_r,@mode)
-      @datafile = File.open(@dset_r,"r")
+#      @datafile = File.open(@dset_r,"r")
+      @datafile = File.open(@dset_r,"rb")
     else
       start_byte = start_byte(name, level, time)
-#      @datafile = File.open(@dset,@mode)
-      @datafile = File.open(@dset,"r")
+#      @datafile = File.open(@dset,"r")
+      @datafile = File.open(@dset,"rb")
+    end
+
+    @x_len = @dimensions[0][:len]
+    @y_len = @dimensions[1][:len]
+
+    if(lonlat)
+      if !(lonlat.is_a?(Array) || lonlat.is_a?(NArray))
+        raise "lonlat must be given Array or NArray"
+      end
+#      p lonlat
+      lon_str = lonlat[0]
+      lon_end = lonlat[1]
+      lat_str = lonlat[2]
+      lat_end = lonlat[3]
+      
+      if( @map[name][:xytranspose] )
+        @datafile.pos = start_byte + @y_len*lon_str*@map[name][:byte]
+        ary = NArray.to_na(@datafile.read(@y_len*(lon_end-lon_str+1)*@map[name][:byte]), 
+                           @map[name][:type], @y_len, lon_end-lon_str+1)
+        ary = ary[lat_str..lat_end,true].transpose
+      else
+        @datafile.pos = start_byte + @x_len*lat_str*@map[name][:byte]
+        ary = NArray.to_na(@datafile.read(@x_len*(lat_end-lat_str+1)*@map[name][:byte]), 
+                           @map[name][:type], @x_len, lat_end-lat_str+1)
+        ary = ary[lon_str..lon_end,true]
+      end
+    else
+      @datafile.pos = start_byte
+#      ary = NArray.to_na(@datafile.read(@xy*@map[name][:byte]), @map[name][:type],
+      ary = NArray.to_na(@datafile.read(@x_len*@y_len*@map[name][:byte]), @map[name][:type],
+                         @dimensions[0][:len],@dimensions[1][:len])
+      ary = ary.transpose if ( @map[name][:xytranspose] )
     end
-    @datafile.pos=start_byte
 
-#    ary = NArray.to_na(@datafile.read(@xybytes), "sfloat",
-#                       @dimensions[0][:len],@dimensions[1][:len])
-    ary = NArray.to_na(@datafile.read(@xy*@map[name][:byte]), @map[name][:type],
-                       @dimensions[0][:len],@dimensions[1][:len])
     ary = ary.ntoh if ( @options["big_endian"] || @options["cray_32bit_ieee"] )
     ary = ary.vtoh if ( @options["little_endian"] )
     if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
@@ -587,8 +624,6 @@ EOS
       raise "endian specification error"
     end
 
-    ary = ary.transpose if ( @map[name][:xytranspose] )
-
     @datafile.close
     ary
   end
@@ -602,7 +637,7 @@ EOS
       if (dim[:levels])
         var_dim = NArray.to_na(dim[:levels])
       elsif (dim[:start] && dim[:increment] && dim[:len])
-        var_dim = NArray.sfloat(dim[:len]).indgen!*dim[:increment]+dim[:start]
+        var_dim = NArray.float(dim[:len]).indgen!*dim[:increment]+dim[:start]
       else
         raise "cannot define dimension "+name
       end
@@ -677,8 +712,8 @@ EOS
         if ($1)
 #         @dset = $1
 #         @dset = $1.gsub(/(\^)/,"./")  # replace "^" with "./"
-         @dset = $1.gsub(/(\^)/,File.dirname(@ctlfile.path)+"/")
-                   # replace "^" with ctlfile directory name
+         @dset = $1.gsub(/(^\^|^([^\/]))/,File.dirname(@ctlfile.path)+'/\2')
+                   # relative -> absolute path (e.g. '^a' or 'a' -> '/hoge/a')
 #         @datafile = File.open(@dset,@mode)
         else
           raise "Invalid line: "+line
@@ -755,8 +790,10 @@ EOS
               if( /^\s*[\d\-\.]/ =~ levs )
                 dim[:levels] += levs.split.collect!{|i| i.to_f}
               end
-              while (dim[:spec] += levs = @ctlfile.gets)
+#              while (dim[:spec] += levs = @ctlfile.gets)
+              while (levs = @ctlfile.gets)
                 if( /^\s*[\d\-\.]/ =~ levs )
+                  dim[:spec] += levs
                   dim[:levels] += levs.split.collect!{|i| i.to_f}
                   #p '###  levels',dim[:levels].length,"  ",levs.split
                   pos = @ctlfile.pos    # back up for a one-line rewind
@@ -782,7 +819,8 @@ EOS
               dim[:increment],dim[:increment_units] = parse_timeincrement(increment)
 #              dim[:units] = dim[:increment_units]
 #              dim[:units] = dim[:start_units]
-              dim[:units] = "days since #{year}-#{mon}-#{day} #{hour}:#{min}:0 +0:00"
+#              dim[:units] = "days since #{year}-#{mon}-#{day} #{hour}:#{min}:0 +0:00"
+              dim[:units] = sprintf("days since #{year}-%02i-%02i %02i:%02iZ",mon,day,hour,min)
             else
               raise "invalid dimension flag(only LINEAR is available for time)"
             end
@@ -868,9 +906,9 @@ EOS
     raise "DSET field is not found" if(!@dset)
     raise "VARS field is not found" if(!@variables)
 
-    #<post processing>
-    @xybytes = 4 * @dimensions[0][:len] * @dimensions[1][:len]
-    @xy = @dimensions[0][:len] * @dimensions[1][:len]
+#    #<post processing>
+#    @xybytes = 4 * @dimensions[0][:len] * @dimensions[1][:len]
+#    @xy = @dimensions[0][:len] * @dimensions[1][:len]
 
   end
 
@@ -887,7 +925,8 @@ EOS
 #      iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
 #      str_byte = map[:offset] + @xybytes*iblock
       iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
-      str_byte = map[:offset] + @xy*iblock
+#      str_byte = map[:offset] + @xy*iblock
+      str_byte = map[:offset] + @dimensions[0][:len] * @dimensions[1][:len] * iblock
       if( @options["sequential"] )
         str_byte += iblock*8 + 4
       end
@@ -900,7 +939,7 @@ EOS
   def parse_starttime(string)
     ## interpret the hh:mmZddmmmyyyy format for grads
     
-    if (/(.*)Z(.*)/ =~ string)
+    if (/([\d:]*)Z(.*)/i =~ string)
       stime = $1
       sdate = $2
     else
@@ -983,9 +1022,8 @@ EOS
         raise "invalid units: "+sunits
       end
     else
-      raise "invalid time-increment string: "+sunits
+      raise "invalid time-increment string: #{sunits}"
     end
-    #print "### ",increment, ' ',units,"\n"
     return [increment, units]
   end
 
@@ -993,6 +1031,58 @@ EOS
     a>b ? a : b
   end
 
+  def generate_starttime(startjd)
+    date = Date.jd(startjd)
+    year, mon, day = date.year, date.mon, date.day
+    hour = startjd*24%24
+    min = startjd*1440%60
+
+    mons = ['jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec']
+
+    return sprintf("%02d:%02dZ%02d",hour,min,day)+mons[mon-1]+sprintf("%04d",year)
+  end
+
+  def generate_timeincrement(increment,units)
+    increment = increment[0] if(increment.class==NArray) 
+    sincrement = increment
+    case units
+    when /min/i
+      sunits = "mn"
+    when /hour/i
+      sunits = "hr"
+    when /day/i
+      sunits = "dy"
+    when /mon/i
+      sunits = "mo"
+    when /year/i
+      sunits = "yr"
+    else
+      raise "invalid time increment units: #{units}"
+    end
+
+    if(sunits == "dy")
+      if( increment < 0.0 )
+        raise "invalid time increment: #{increment}"
+      elsif( increment >= 365 && (increment.to_f/365) / (increment.to_i/365) < 1.003 )
+        sincrement = (increment/365).to_i
+        sunits = "yr"
+      elsif( increment >= 28 &&  (increment.to_f/28) / (increment.to_i/28) < 1.11 )
+        sincrement = (increment/28).to_i
+        sunits = "mo"
+      elsif( increment < 1.0 )
+        if( increment >= 0.0416 && (increment*1440)%60 < 5 )
+          sincrement = (increment*24).to_i
+          sunits = "hr"
+        else
+          sincrement = increment*1440
+          sunits = "mn"
+        end
+      end
+    end
+
+    return sprintf("%2d",sincrement)+sunits
+  end
+
   def initialize_new # obsolete
 
     raise "Sorry, this method is far from completed"
@@ -1046,12 +1136,16 @@ class GrADSVar
     else
       @dimensions = @ctl.dimensions
       @rank = @dimensions.length
+      found = false
       for i in 0...@ctl.variables.length
         if @varname == @ctl.variables[i][:name]
           @attr[:long_name] = @ctl.variables[i][:description]
           @attr[:nlev] = @ctl.variables[i][:nlev].to_s
+          @attr[:missing_value] = NArray.sfloat(1).fill!(@ctl.undef)
+	  found = true
         end
       end
+      raise "variable #{@varname} is not found" if !found
     end
   end
 
@@ -1087,7 +1181,7 @@ class GrADSVar
         end
       end
     else
-      type = "sfloat"
+      type = "float"
     end
     type
   end
@@ -1219,23 +1313,19 @@ class GrADSVar
       str_y = idx_y[h_sta[1]]; end_y = idx_y[h_end[1]]
       str_z = idx_z[h_sta[2]]; end_z = idx_z[h_end[2]]
       str_t = idx_t[h_sta[3]]; end_t = idx_t[h_end[3]]
-#      p str_x,end_x,str_y,end_y,str_z,end_z,str_t,end_t
 
+      na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
       for time in str_t..end_t
         for lev in str_z..end_z
-#          na_xy = @ctl.get(@varname,lev,time).swap_byte
-          na_xy = @ctl.get(@varname,lev,time)
-          if time == str_t && lev == str_z
-#            na = NArray.sfloat(end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
-#            na = NArrayMiss.sfloat(end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
-            na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
-          end
-#          na[true,true,lev-str_z,time-str_t] = na_xy[str_x..end_x,str_y..end_y]
-#          mask = ( na_xy[str_x..end_x,str_y..end_y].abs < @ctl.undef.abs/2 )
-          mask = ( na_xy[str_x..end_x,str_y..end_y].ne @ctl.undef )
-#          mask = ( na_xy[str_x..end_x,str_y..end_y] > -100.0 )
-          na[true,true,lev-str_z,time-str_t] = NArrayMiss.to_nam(na_xy[str_x..end_x,str_y..end_y],mask)
-#          na[true,true,lev-str_z,time-str_t].set_mask(mask)
+#          na_xy = @ctl.get(@varname,lev,time)
+##          if time == str_t && lev == str_z
+##            na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
+##          end
+#          mask = ( na_xy[str_x..end_x,str_y..end_y].ne @ctl.undef )
+#          na[true,true,lev-str_z,time-str_t] = NArrayMiss.to_nam(na_xy[str_x..end_x,str_y..end_y],mask)
+          na_xy = @ctl.get(@varname,lev,time,[str_x,end_x,str_y,end_y])
+          mask = ( na_xy.ne @ctl.undef )
+          na[true,true,lev-str_z,time-str_t] = NArrayMiss.to_nam(na_xy,mask)
         end
       end
     end
@@ -1245,6 +1335,160 @@ class GrADSVar
   alias simple_put put
   alias simple_get get
 
+   def __rubber_expansion( args )  # copied from NetCDFVar class
+     if (id = args.index(false))  # substitution into id
+       # false is incuded
+       alen = args.length
+       if args.rindex(false) != id
+         raise ArguemntError,"only one rubber dimension is permitted"
+       elsif alen > rank+1
+         raise ArgumentError, "too many args"
+       end
+       ar = ( id!=0 ? args[0..id-1] : [] )
+       args = ar + [true]*(rank-alen+1) + args[id+1..-1]
+     elsif args.length == 0   # to support empty [], []=
+       args = [true]*rank
+     end
+     args
+   end
+   private :__rubber_expansion
+
+   def [](*a)  # copied from NetCDFVar class
+     a = __rubber_expansion(a)
+     first = Array.new
+     last = Array.new
+     stride = Array.new
+     set_stride = false
+     a.each{|i|
+       if(i.is_a?(Fixnum))
+         first.push(i)
+         last.push(i)
+         stride.push(1)
+       elsif(i.is_a?(Range))
+         first.push(i.first)
+         last.push(i.exclude_end? ? i.last-1 : i.last)
+         stride.push(1)
+       elsif(i.is_a?(Hash))
+         r = (i.to_a[0])[0]
+         s = (i.to_a[0])[1]
+         if ( !( r.is_a?(Range) ) || ! ( s.is_a?(Integer) ) )
+            raise TypeError, "Hash argument must be {a_Range, step}"
+         end
+         first.push(r.first) 
+         last.push(r.exclude_end? ? r.last-1 : r.last)
+         stride.push(s)
+         set_stride = true
+       elsif(i.is_a?(TrueClass))
+         first.push(0)
+         last.push(-1)
+         stride.push(1)
+       elsif( i.is_a?(Array) || i.is_a?(NArray))
+         a_new = a.dup
+         at = a.index(i)
+         i = NArray.to_na(i) if i.is_a?(Array)
+         for n in 0..i.length-1
+           a_new[at] = i[n]..i[n]
+           na_tmp = self[*a_new]
+           if n==0 then
+             k = at
+             a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+             shape_tmp = na_tmp.shape
+             shape_tmp[k] = i.length
+             na = na_tmp.class.new(na_tmp.typecode,*shape_tmp)
+             index_tmp = Array.new(shape_tmp.length,true)
+           end
+           index_tmp[k] = n..n
+           na[*index_tmp] = na_tmp
+         end
+         return na
+       else
+         raise TypeError, "argument must be Fixnum, Range, Hash, TrueClass, Array, or NArray"
+       end
+     }
+
+     if(set_stride)
+       na = self.get({"start"=>first, "end"=>last, "stride"=>stride})
+     else
+       na = self.get({"start"=>first, "end"=>last})
+     end
+     shape = na.shape
+     (a.length-1).downto(0){ |i|
+         shape.delete_at(i) if a[i].is_a?(Fixnum)
+      }
+      na.reshape!( *shape )
+     na
+   end
+
+   def []=(*a)  # copied from NetCDFVar class
+     val = a.pop
+     a = __rubber_expansion(a)
+     first = Array.new
+     last = Array.new
+     stride = Array.new
+     set_stride = false
+     a.each{|i|
+       if(i.is_a?(Fixnum))
+         first.push(i)
+         last.push(i)
+         stride.push(1)
+       elsif(i.is_a?(Range))
+         first.push(i.first)
+         last.push(i.exclude_end? ? i.last-1 : i.last)
+         stride.push(1)
+       elsif(i.is_a?(Hash))
+         r = (i.to_a[0])[0]
+         s = (i.to_a[0])[1]
+         if ( !( r.is_a?(Range) ) || ! ( s.is_a?(Integer) ) )
+            raise ArgumentError, "Hash argument must be {first..last, step}"
+         end
+         first.push(r.first) 
+         last.push(r.exclude_end? ? r.last-1 : r.last)
+         stride.push(s)
+         set_stride = true
+       elsif(i.is_a?(TrueClass))
+         first.push(0)
+         last.push(-1)
+         stride.push(1)
+       elsif(i.is_a?(Array) || i.is_a?(NArray))
+         a_new = a.dup
+         at = a.index(i)
+         i = NArray.to_na(i) if i.is_a?(Array)
+         val = NArray.to_na(val) if val.is_a?(Array)
+         rank_of_subset = a.dup.delete_if{|v| v.is_a?(Fixnum)}.length
+         if val.rank != rank_of_subset
+           raise "rank of the rhs (#{val.rank}) is not equal to the rank "+
+                 "of the subset specified by #{a.inspect} (#{rank_of_subset})"
+         end
+         k = at
+         a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+         if i.length != val.shape[k]
+           raise "length of the #{k+1}-th dim of rhs is incorrect "+
+                 "(#{i.length} for #{val.shape[k]})"
+         end
+         index_tmp = Array.new(val.rank,true) if !val.is_a?(Numeric) #==>Array-like
+         for n in 0..i.length-1
+           a_new[at] = i[n]..i[n]
+           if !val.is_a?(Numeric) then
+             index_tmp[k] = n..n
+             self[*a_new] = val[*index_tmp]
+           else
+             self[*a_new] = val
+           end
+         end
+         return self
+       else
+         raise TypeError, "argument must be Fixnum, Range, Hash, TrueClass, Array, or NArray"
+       end
+     }
+
+     if(set_stride)
+        self.put(val, {"start"=>first, "end"=>last, "stride"=>stride})
+     else
+        self.put(val, {"start"=>first, "end"=>last})
+     end
+   end
+
+=begin
    def [](*a)  # copied from NetCDFVar class
      first = Array.new
      last = Array.new
@@ -1327,6 +1571,7 @@ class GrADSVar
         self.put(val, {"start"=>first, "end"=>last})
      end
    end
+=end
 
 end
 
