@@ -1,9 +1,8 @@
 =begin
 =Status
-* only reading is supported
 * only a Lon-Lat coordinate is supported
-* only a number of bits for packing which is multiple of 4 is allowed
 * only simple pakking is supported
+* variable list is not complete for NCEP and ECMWF
 =end
 require "narray_miss"
 require "date"
@@ -42,6 +41,14 @@ module NumRu
       str.length==len || raise("str length is not valid")
       return str
     end
+    def float_value( str )
+      str.length==4 || raise("str length must be 4")
+      sa = str2int( str[0..0] )
+      s = (sa<0) ? -1 : 1
+      a = sa.abs
+      b = str2uint( str[1..3] )
+      return s*(2.0)**(-24.0)*b*(16.0)**(a-64.0)
+    end
     def get_bits( str, nstep, nbits, sbu, ebu, type )
       len = str.length
       bx = NArray.to_na(str,NArray::BYTE)
@@ -75,7 +82,7 @@ module NumRu
         x[6,true] = (bx&2)/2
         x[7,true] = bx&1
         x.reshape!(len*8)
-        y.fill(1)
+        y = 2**y
       end
       x = x[sbu..-1-ebu]
       x.reshape!(nb,x.length/nb)
@@ -224,7 +231,7 @@ module NumRu
           else
             raise "not defined yet"
           end
-        elsif x.att("long_name").downcase=="longitude" && y.att("long_name").downcase=="latitude"
+        elsif y.att("long_name").downcase=="longitude" && x.att("long_name").downcase=="latitude"
           xv = y.val
           yv = x.val
           rel = true
@@ -270,7 +277,7 @@ module NumRu
         end
         pds.set_initial(ye,mo,da,ho,mi)
       end
-      def set_var(var,name)
+      def set_var(var,name,units)
         is.set_version(1)
         pds.set_version(2)
         pds.set_center_id(0)
@@ -296,6 +303,10 @@ module NumRu
         end
         bds.set_grid(true)
         bds.set_simple(true)
+        if units && pds.unit
+          f,o = Units[units].factor_and_offset(Units[pds.unit])
+          var = var*f+o
+        end
         if var.typecode==NArray::SINT || var.typecode==NArray::INT
           bds.set_ingeger(true)
         else
@@ -358,9 +369,15 @@ module NumRu
         when 1
           params = PARAMS_1
         when 2,3
-          params = PARAMS_2
+          if nid<128
+            params = PARAMS_2
           else
-          if cid==98
+            if cid==7 # NCEP
+              params = PARAMS_NCEP_2
+            end
+          end
+        else
+          if cid==98 # ECMWF
             if version==128
               params = PARAMS_ECMWF_128
             end
@@ -613,17 +630,8 @@ module NumRu
       def gtype
         return str2uint( @gds[2,1] )
       end
-      def list
-        if gtype==3||gtype==1
-          return @gds[39..-1]
-        elsif gtype==90
-          return @gds[41..-1]
-        else
-          return @gds[29..-1]
-        end
-      end
       def get_x_y(grid)
-        case gtype
+      case gtype
         when 0,4
           #lon_lat (+GussLat)
           flag = str2uint( grid[10,1] )
@@ -695,7 +703,7 @@ module NumRu
             (vlat[0]*1000).round==la1.round || $stderr.printf("Warning: first of latitude is not same: #{vlat[0]} != #{la1.to_f/1000}\n")
             (vlat[-1]*1000).round==la2.round || $stderr.printf("Warning: last of latitude is not same: #{vlat[-1]} != #{la2.to_f/1000}\n")
             lat["type"] = "Gaussian latitude"
-            lat["number of latitude circles"] = dlat*2
+            lat["number_of_latitude_circles"] = NArray[dlat*2]
           end
           if flag_scanj==1
             vlat = vlat[-1..0]
@@ -813,8 +821,8 @@ module NumRu
       end
       def pl
         pl = str2uint( @gds[1,1] )
-        return nil if pl==255 || nv!=0
-        return pl
+        return nil if pl==255
+        return nv*4+pl
       end
       def grid_type
         return GRID_TYPES[ gtype ]
@@ -831,65 +839,57 @@ module NumRu
       end
       def set_grid(id,x,y,rev)
         @gds[2..2] = uint2str(id,1)
-        li = list
         if id==3||id==1
           str = grid2str(id,x,y,rev)
           str.length==36 || raise("length is not correct")
-          list = @gds[39..-1]
-          @gds = @gds[0..2]<<str
-          @gds<<li if li
+          @gds[3..38] = str
           @sgm.is.update_total_length
         elsif id==90
           str = grid2str(id,x,y,rev)
           str.length==38 || raise("length is not correct")
-          @gds = @gds[0..2]<<str
-          @gds<<li if li
+          @gds[3..40] = str
           @sgm.is.update_total_length
         else
           str = grid2str(id,x,y,rev)
           str.length==26 || raise("length is not correct")
-          @gds = @gds[0..2]<<str
-          @gds<<li if li
+          @gds[3..28] = str
           @sgm.is.update_total_length
         end
         return [id,x,y]
       end
       def list_pv
-        return list if pv
-        return nil
+        return nil unless pv
+        list = NArray.float(nv)
+        nv.times{|n|
+          list[n] = float_value( @gds[pv-4+n*4...pv-4+(n+1)*4] )
+        }
+        return list
       end
       def set_list_pv(list)
-        nv = list.length
+        nv = list.length/4
         @gds[0..0] = uint2str(nv,1)
         raise "not defined yet"
-#        pv = 
-#        @gds[1..1] = uint2str(pv,1)
         if gtype==3||gtype==1
-          @gds[39..-1] = list
+          pv = 43
         elsif gtype==90
-          @gds[41..-1] = list
+          pv = 45
         else
-          @gds[29..-1] = list
+          pv = 33
         end
+        @gds[1..1] = uint2str(pv,1)
+        @gds[pv-4..-1] = list
         return true
       end
       alias :list_pv= :set_list_pv
       def list_pl
-        return list if pl
+        return @gds[pl-4..-1] if pl
         return nil
       end
-      def set_list_pl
-        @gds[0..0] = "\001"
+      def set_list_pl(list)
         raise "not defined yet"
-#        pvl= 
-#        @gds[1..1] = uint2str(pl,1)
-        if gtype==3||gtype==1
-          @gds[39..-1] = list
-        elsif gtype==90
-          @gds[41..-1] = list
-        else
-          @gds[29..-1] = list
-        end
+        pl = nv*4+pv
+        @gds[1..1] = uint2str(pl,1)
+        @gds[pl-4..-1] = list
         return true
       end
       alias :list_pv= :set_list_pv
@@ -1126,11 +1126,7 @@ module NumRu
       def offset
         return @offset if @offset
         @sgm.file.seek(@pos+3, IO::SEEK_SET)
-        sa = str2int( @sgm.file.read(1) )
-        s = (sa<0) ? -1 : 1
-        a = sa.abs
-        b = str2uint( @sgm.file.read(3) )
-        @offset = s*(2.0)**(-24.0)*b*(16.0)**(a-64.0)
+        @offset = float_value(@sgm.file.read(4))
         return @offset
       end
       def efactor
@@ -1148,9 +1144,9 @@ module NumRu
         elsif (nbits_pack%4) == 0
           nstep = 4
         elsif (nbits_pack%2) == 0
-          nstep == 2
+          nstep = 2
         else
-          nstep == 1
+          nstep = 1
         end
         r = offset
         e = efactor
@@ -1267,13 +1263,17 @@ module NumRu
     end # end definition of class GribES
 
     def initialize(fname,mode="r")
-      if mode!="r"&&mode!="w"
-        raise "only 'r' and 'w' are allowed for mode"
+      case mode
+      when "r","rb"
+	mode = "rb"
+      when "w","wb"
+	mode = "wb"
+      else
+        raise "Unsupported file mode '#{mode}'. Use 'r' or 'w'."
       end
       @file = File.open(fname, mode)
       @vars = Hash.new
       @attr = Hash.new
-      @mode = mode=="w"
     end
 
     public
@@ -1284,6 +1284,10 @@ module NumRu
 ---Grib.new(filename, mode="r")
    make a new Grib object.
 
+   ARGUMENTS
+   * filename (String): name of the file to open
+   * mode (String): IO mode. "r" (read only) or "w" (write only).
+
 ---Grib.open(filename)
    open a Grib file.
 
@@ -1293,12 +1297,6 @@ module NumRu
 ---Grib.is_aGrib?(filename)
    return true when file is a Grib dataset
 
-   ARGUMENTS
-   * filename (String): name of the file to open
-   * mode (String): IO mode. "r" or "w+" (precisely speaking, you can
-     use "w", but it is interpreted as "w+". Thus, all files opened
-     will be readable). CURRENTLY, ONLY "r" IS HANDLED.
-
 ==Methods
 ---close
 ---parse
@@ -1313,7 +1311,7 @@ module NumRu
       def is_a_Grib?(fname)
         file = nil
         begin
-          file = File.open(fname,"r")
+          file = File.open(fname,"rb")
           is = (file.read(4) == "GRIB")
         ensure
           file.close if file
@@ -1364,8 +1362,8 @@ module NumRu
       return GribVar.parse(self,var,name)
     end
     def def_var(name)
-      if !@mode || !@file.stat.writable?
-        raise("cannot defind variable to #{@file.path}")
+      until @file.stat.writable?
+        raise("File #{@file.path} is not writable.")
       end
       if !String===name
         raise "def_var(String)"
@@ -1381,9 +1379,6 @@ module NumRu
     def att(name)
       @attr[name]
     end
-    def enddef
-      @mode = false
-    end
     def write
       iz=0
       it=0
@@ -1396,7 +1391,7 @@ module NumRu
           sgm = nil
           if var.rank==2
             sgm = GribSegment.create(@file)
-            val = [var.get,var.att("long_name")]
+            val = [var.get,var.att("long_name"),var.att("units")]
             z = var.att("level")
             t = var.att("time")
             vn.delete(name)
@@ -1405,7 +1400,7 @@ module NumRu
             izmax < sha[2] && izmax = sha[2]
             if iz<sha[2]&&it<sha[3]
               sgm = GribSegment.create(@file)
-              val = [var[true,true,iz,it],var.att("long_name")]
+              val = [var[true,true,iz,it],var.att("long_name"),var.att("units")]
               zd = var.dim(2)
               zv = zd.get
               if Array===zv
@@ -1413,7 +1408,7 @@ module NumRu
               else
                 zv = [zv[iz]]
               end
-              z = [zd["long_name"], zv]
+              z = [zd.att("long_name"), zv]
               t = var.dim(3)[it]
             elsif it==sha[3]
               vn.delete(name)
@@ -1423,7 +1418,7 @@ module NumRu
             izmax < sha[2] && izmax = sha[2]
             if iz<sha[2]
               sgm = GribSegment.create(@file)
-              val = [var[true,true,iz],var.att("long_name")]
+              val = [var[true,true,iz],var.att("long_name"),var.att("units")]
               zd = var.dim(2)
               zv = zd.get
               if Array===zv
@@ -1439,7 +1434,7 @@ module NumRu
             sha = var.shape
             if it<sha[2]
               sgm = GribSegment.create(@file)
-              val = [var[true,true,it],var.att("long_name")]
+              val = [var[true,true,it],var.att("long_name"),var.att("units")]
               z = var.att("level")
               t = var.dim(2)[it]
             else
@@ -1514,7 +1509,15 @@ module NumRu
       end
     end
     def val
-      return Array===@ary ? @ary[0]["value"] : @ary
+      if Array===@ary
+        if att("long_name")=="Hybrid level"
+          return @ary[0]["value"]
+        else
+          return @ary[1]["value"]
+        end
+      else
+        return @ary
+      end
     end
     def [](*ind)
       return val[*ind]
@@ -1623,7 +1626,7 @@ module NumRu
         va.put_att("long_name",pds.name)
         std_name = pds.standard_name
         va.put_att("standard_name",std_name) if std_name
-        va.put_att("units",pds.unit)
+        va.put_att("units",pds.unit) if pds.unit
         va.set_miss(miss)
         vdim = Array.new
         x,y = gds.grid
@@ -1643,7 +1646,7 @@ module NumRu
             d.put_att(k,v)
           end
         }
-        if sgms.length>1
+        if sgms.length>=1
           nz = pds.z_value.length
           if nz!=0
             z = Array.new(nz).collect{Array.new}
@@ -1660,21 +1663,44 @@ module NumRu
               z[m].length==z[0].length || raise("length is not same")
             }
             if z[0].length>1
-              nz.times{|m|
-                zval = pds.z_value[m]
-                z[m] = {"value"=>NArray.to_na(z[m]),"name"=>zval["name"],"units"=>zval["units"]}
-              }
               d = va.def_dim(pds.z_sname,2)
-              d.put(z)
               d.put_att("long_name",pds.z_type)
-              unit = pds.z_value[0]["units"]
-              d.put_att("units",unit) if unit
+              if pds.z_type=="Hybrid level"
+                list_pv = gds.list_pv
+                zval = pds.z_value[0]
+                tmp = Array.new
+                z[0].each{|n|
+                  tmp += [n-1,n]
+                }
+                tmp.uniq!
+                tmp.sort!
+                ap = list_pv[0...list_pv.length/2]
+                b = list_pv[list_pv.length/2..-1]
+                z[0] = {"value"=>NArray.to_na(z[0]),"name"=>zval["name"]}
+                z[1] = {"value"=>ap[tmp], "name"=>"ap_half_lev", "long_name"=>"hybrid ap at half levels", "units"=>"Pa"}
+                z[2] = {"value"=>b[tmp], "name"=>"b_half_lev", "long_name"=>"hybrid b at half levels", "units"=>"1"}
+              elsif nz==1
+                zval = pds.z_value[0]
+                z = NArray.to_na(z[0])
+                d.put_att("units",zval["units"])
+                d.put_att("value_type",zval["name"])
+              else
+                nz.times{|mm|
+                  m = nz-m-1
+                  zval = pds.z_value[m]
+                  z[m+1] = {"value"=>NArray.to_na(z[m]),"name"=>zval["name"],"units"=>zval["units"]}
+                }
+                z[0] = {"value"=>NArray.sint(z[0].length).indgen,"name"=>"level number"}
+              end
+              d.put(z)
+            else
+              va.put_att("level", pds.z_type)
+              pds.z_value.each{|zz|
+                va.put_att(zz["name"].gsub(/\s/,"_"), "#{zz["value"]} #{zz["units"]}")
+              }
             end
           else
             va.put_att("level", pds.z_type)
-            pds.z_value.each{|zz|
-              va.put_att("#{zz["name"]} (#{zz["units"]})", zz["value"])
-            }
           end
           time = Array.new
           sgms.each{|sgm|
@@ -1807,11 +1833,13 @@ module NumRu
           if rank>2
             mask = NArray.byte(*shape[2..-1])
             mask[*indices[2..-1]]= 1
-            first = indices[2..-1].collect{|i|
-              if Fixnum===i
-                i
-              elsif Range===i
-                i.first
+            first = Array.new(indices.length-2)
+	    first.length.times{|i|
+              ind = indices[2+i]
+              if Fixnum===ind
+	        first[i] = ind < 0 ? shape[2+i]+ind : ind
+              elsif Range===ind
+                first[i] = ind.first
               else
                 raise "invalid indices"
               end
@@ -1825,7 +1853,7 @@ module NumRu
         value = @miss ? NArrayMiss.float(*sha) : NArray.float(*sha)
         if rank==2
           vz = nil
-          vt = nil
+         vt = nil
           index = []
         elsif rank==4
           vz = dim(2).val
@@ -1904,6 +1932,8 @@ if $0 == __FILE__
   grib.var_names.each{|vn|
     p v = grib.var(vn)
     p v.dim_names
+    v.dim_names.each{|dn| p dn; p v.dim(dn).get }
+    p v.shape
     v.att_names.each{|an| print an, " => ", v.att(an), "\n" }
     puts "\n"
   }
