@@ -64,13 +64,17 @@ Z_TYPES = {
   112 => ["layer between two depths below land surface", "lbls"],
   113 => ["isentropic (theta) level", "isen"],
   114 => ["layer between two isentropic levels", "lisn"],
+  116 => ["layer between two isobaric surfaces above ground", "lisg"],
   121 => ["layer between two isobaric surfaces (high precision)", "lish"],
   125 => ["Height level above ground (heigh precision)", "fhgh"],
   128 => ["layer between two sigma levels (high precision)", "lsh"],
   141 => ["layer between two isobaric surfaces (mixed precision)", "lism"],
   160 => ["depth below sea level", "dbs"],
   200 => ["entire atmosphere considered as a single layer", "atm"],
-  201 => ["entire ocean considered as single layer", "ocn"]
+  201 => ["entire ocean considered as single layer", "ocn"],
+  242 => ["convect cloud bottom", "ccb"],
+  243 => ["convect cloud top", "cct"],
+  244 => ["convect cloud layer", "ccl"],
 }
 Z_LEVELS = {
   100 => [{"name"=>"pressure","units"=>"hPa"}],
@@ -94,6 +98,8 @@ Z_LEVELS = {
   113 => [{"name"=>"potential temperature","units"=>"K"}],
   114 => [{"name"=>"475K minus theta of top","units"=>"K"},
     {"name"=>"475K minus theta of bottom","units"=>"K"}],
+  116 => [{"name"=>"pressure of top above ground", "units"=>"hPa"},
+    {"name"=>"pressure of bottom above ground", "units"=>"hPa"}],
   121 => [{"name="=>"1100hPa minus pressure of top","units"=>"hPa"},
     {"name"=>"1100hPa minus pressure of bottom","units"=>"hPa"}],
   125 => [{"name"=>"height","units"=>"cm"}],
@@ -118,6 +124,7 @@ D19000101 = Date.new(1900,1,1)
 
 
 ## Paramete Constants
+PARAM_UNKNOWN = 0
 PARAM_PRESSURE = 1
 PARAM_PMSL = 2
 PARAM_PTND = 3
@@ -127,7 +134,7 @@ PARAM_GPT = 6
 PARAM_GPT_HGT = 7
 PARAM_GEOM_HGT = 8
 PARAM_HSTDV = 9
-#PARAM_TOZNE = 10
+PARAM_TOZNE = 10
 PARAM_TEMP = 11
 PARAM_VTEMP = 12
 PARAM_POT_TEMP = 13
@@ -275,141 +282,154 @@ PARAM_PRESSURE_DEP = 1023
 PARAM_LAST_ENTRY = 1024
 PARAM_LN_PRES = 1025
 
+## NCEP
+PARAM_LFT_X = 2131
+PARAM_4LFTX = 2132
+PARAM_VW_SH = 2136
+PARAM_SOILW = 2144
+PARAM_O3MR = 2154
+PARAM_CIN = 2156
+PARAM_CAPE = 2157
+PARAM_HPBL = 2221
+PARAM_5WAVH = 2222
+PARAM_5WAVA = 2230
+
 ## ECMWF
-PARAM_EQPT_TEMP = 2004
-PARAM_SEPT_TEMP = 2005
-PARAM_U_DIV_WND = 2011
-PARAM_V_DIV_WND = 2012
-PARAM_U_ROT_WND = 2013
-PARAM_V_ROT_WND = 2014
-PARAM_UBC_TEMP = 2021
-PARAM_UB_PRES = 2022
-PARAM_UB_DIV = 2023
-PARAM_LAKE_COV = 2026
-PARAM_LVEG_COV = 2027
-PARAM_HVEG_COV = 2028
-PARAM_TYP_LVEG = 2029
-PARAM_TYP_HVEG = 2030
-PARAM_ICE_COV = 2031
-PARAM_SNOW_ALBD = 2032
-PARAM_SNOW_DENS = 2033
-PARAM_SEASRF_TEMP = 2034
-PARAM_ICESRF_TEMP = 2035
-PARAM_SOIL_WATER = 2039
-PARAM_TYP_SOIL = 2043
-PARAM_SNOW_EVAP = 2044
+PARAM_EQPT_TEMP = 3004
+PARAM_SEPT_TEMP = 3005
+PARAM_U_DIV_WND = 3011
+PARAM_V_DIV_WND = 3012
+PARAM_U_ROT_WND = 3013
+PARAM_V_ROT_WND = 3014
+PARAM_UBC_TEMP = 3021
+PARAM_UB_PRES = 3022
+PARAM_UB_DIV = 3023
+PARAM_LAKE_COV = 3026
+PARAM_LVEG_COV = 3027
+PARAM_HVEG_COV = 3028
+PARAM_TYP_LVEG = 3029
+PARAM_TYP_HVEG = 3030
+PARAM_ICE_COV = 3031
+PARAM_SNOW_ALBD = 3032
+PARAM_SNOW_DENS = 3033
+PARAM_SEASRF_TEMP = 3034
+PARAM_ICESRF_TEMP = 3035
+PARAM_SOIL_WATER = 3039
+PARAM_TYP_SOIL = 3043
+PARAM_SNOW_EVAP = 3044
 
-PARAM_SOL_DUR = 2045
-PARAM_RND_SOL_DIR = 2047
-PARAM_STRESS_SRF = 2048
-PARAM_WNDGUST = 2049
-PARAM_PRECIP_LS_FRAC = 2050
+PARAM_SOL_DUR = 3045
+PARAM_RND_SOL_DIR = 3047
+PARAM_STRESS_SRF = 3048
+PARAM_WNDGUST = 3049
+PARAM_PRECIP_LS_FRAC = 3050
 
-PARAM_MNTPOT = 2053
+PARAM_MNTPOT = 3053
 
-PARAM_POT_VOR = 2060
+PARAM_POT_VOR = 3060
 
-PARAM_ATM_TIDE = 2127
-PARAM_BUDG_VAL = 2128
+PARAM_ATM_TIDE = 3127
+PARAM_BUDG_VAL = 3128
 
-PARAM_WATER_TOT = 2136
-PARAM_WATV_TOT = 2137
+PARAM_WATER_TOT = 3136
+PARAM_WATV_TOT = 3137
 
-PARAM_SOIL_WET = 2140
+PARAM_SOIL_WET = 3140
 
-PARAM_SNOW_FALL = 2144
+PARAM_SNOW_FALL = 3144
 
-PARAM_CHNK = 2148
-PARAM_RDN_SRF = 2149
-PARAM_RDN_TOP = 2150
+PARAM_CHNK = 3148
+PARAM_RDN_SRF = 3149
+PARAM_RDN_TOP = 3150
 
-PARAM_HT_SW = 2153
-PARAM_HT_LW = 2154
+PARAM_HT_SW = 3153
+PARAM_HT_LW = 3154
 
-PARAM_BL_HGT = 2159
-PARAM_ORG_STDV = 2160
-PARAM_SGORG_ANIS = 2161
-PARAM_SGORG_ANGL = 2162
-PARAM_SGORG_SLOP = 2163
+PARAM_BL_HGT = 3159
+PARAM_ORG_STDV = 3160
+PARAM_SGORG_ANIS = 3161
+PARAM_SGORG_ANGL = 3162
+PARAM_SGORG_SLOP = 3163
 
-PARAM_RND_SOL_SRFD = 2169
+PARAM_RND_SOL_SRFD = 3169
 
-PARAM_ALBEDO_F = 2174
-PARAM_RDN_THE_SRFD = 2175
-PARAM_RDN_SOL_SRF = 2176
-PARAM_RDN_THE_SRF = 2177
-PARAM_RDN_SOL_TOP = 2178
-PARAM_RDN_THE_TOP = 2179
-PARAM_STRESS_EW_SRF = 2180
-PARAM_STRESS_NS_SRF = 2181
+PARAM_ALBEDO_F = 3174
+PARAM_RDN_THE_SRFD = 3175
+PARAM_RDN_SOL_SRF = 3176
+PARAM_RDN_THE_SRF = 3177
+PARAM_RDN_SOL_TOP = 3178
+PARAM_RDN_THE_TOP = 3179
+PARAM_STRESS_EW_SRF = 3180
+PARAM_STRESS_NS_SRF = 3181
 
-PARAM_SUN_DUR = 2189
-PARAM_ORG_EW_VARI = 2190
-PARAM_ORG_NS_VARI = 2191
-PARAM_ORG_NWSE_VARI = 2192
-PARAM_ORG_NESW_VARI = 2193
-PARAM_BRIT_TEMP = 2194
-PARAM_STRESS_LAT_GWAVE = 2195
-PARAM_STRESS_MER_GWAVE = 2196
-PARAM_GW_DISS  = 2197
-PARAM_SKIN_RESV = 2198
-PARAM_VEG_FRAC = 2199
-PARAM_ORG_VARI = 2200
+PARAM_SUN_DUR = 3189
+PARAM_ORG_EW_VARI = 3190
+PARAM_ORG_NS_VARI = 3191
+PARAM_ORG_NWSE_VARI = 3192
+PARAM_ORG_NESW_VARI = 3193
+PARAM_BRIT_TEMP = 3194
+PARAM_STRESS_LAT_GWAVE = 3195
+PARAM_STRESS_MER_GWAVE = 3196
+PARAM_GW_DISS  = 3197
+PARAM_SKIN_RESV = 3198
+PARAM_VEG_FRAC = 3199
+PARAM_ORG_VARI = 3200
 
-PARAM_OZON_MIX = 2203
-PARAM_PRECIP_WGT = 2204
-PARAM_RUNOFF = 2205
-PARAM_OZON_TOT = 2206
+PARAM_OZON_MIX = 3203
+PARAM_PRECIP_WGT = 3204
+PARAM_RUNOFF = 3205
+PARAM_OZON_TOT = 3206
 
-PARAM_RDN_SOL_CS_TOP = 2208
-PARAM_RDN_THE_CS_TOP = 2209
-PARAM_RDN_SOL_CS_SRF = 2210
-PARAM_RDN_THE_CS_SRF = 2211
-PARAM_SOL_INSL = 2212
+PARAM_RDN_SOL_CS_TOP = 3208
+PARAM_RDN_THE_CS_TOP = 3209
+PARAM_RDN_SOL_CS_SRF = 3210
+PARAM_RDN_THE_CS_SRF = 3211
+PARAM_SOL_INSL = 3212
 
-PARAM_DHT_RDN = 2214
-PARAM_DHT_VDIFS = 2215
-PARAM_DHT_CUMCONV = 2216
-PARAM_DHT_LSCND = 2217
-PARAM_VDIFS_ZWND = 2218
-PARAM_VDIFS_MWND = 2219
-PARAM_DTND_EW_GWAVE = 2220
-PARAM_DTND_NS_GWAVE = 2221
-PARAM_CTND_ZWND = 2222
-PARAM_CTND_MWND = 2223
-PARAM_VDIFS_HUM = 2224
-PARAM_HTND_CUMCONV = 2225
-PARAM_HTND_LSCND = 2226
-PARAM_CNG_NHUM = 2227
+PARAM_DHT_RDN = 3214
+PARAM_DHT_VDIFS = 3215
+PARAM_DHT_CUMCONV = 3216
+PARAM_DHT_LSCND = 3217
+PARAM_VDIFS_ZWND = 3218
+PARAM_VDIFS_MWND = 3219
+PARAM_DTND_EW_GWAVE = 3220
+PARAM_DTND_NS_GWAVE = 3221
+PARAM_CTND_ZWND = 3222
+PARAM_CTND_MWND = 3223
+PARAM_VDIFS_HUM = 3224
+PARAM_HTND_CUMCONV = 3225
+PARAM_HTND_LSCND = 3226
+PARAM_CNG_NHUM = 3227
 
-PARAM_STRESS_EW_SRF_IN = 2229
-PARAM_STRESS_NS_SRF_IN = 2230
-PARAM_HT_SRF_IN = 2231
-PARAM_MST_IN = 2232
-PARAM_APP_HUM_SRF = 2233
-PARAM_LRGN_HT_SRF = 2234
-PARAM_SKIN_TEMP = 2235
+PARAM_STRESS_EW_SRF_IN = 3229
+PARAM_STRESS_NS_SRF_IN = 3230
+PARAM_HT_SRF_IN = 3231
+PARAM_MST_IN = 3232
+PARAM_APP_HUM_SRF = 3233
+PARAM_LRGN_HT_SRF = 3234
+PARAM_SKIN_TEMP = 3235
 
-PARAM_SNOW_TEMP = 2238
-PARAM_SNOW_FALL_CONV = 2239
-PARAM_SNOW_FALL_LS = 2240
-PARAM_CLOUD_TND = 2241
-PARAM_LW_TND = 2242
-PARAM_ALBEDO_FC = 2243
-PARAM_RGN_SRF_FC = 2244
-PARAM_RGN_NT_SRF_FC = 2245
-PARAM_CLOUD_LIQ = 2246
-PARAM_CLOUD_ICE = 2247
-PARAM_CLOUD_F = 2248
-PARAM_ICE_TND = 2249
-PARAM_ICE_AGE = 2250
-PARAM_TND_TEMP_AD = 2251
-PARAM_TND_HUM_AD = 2252
-PARAM_TND_ZWND_AD = 2253
-PARAM_TND_MWND_AD = 2254
+PARAM_SNOW_TEMP = 3238
+PARAM_SNOW_FALL_CONV = 3239
+PARAM_SNOW_FALL_LS = 3240
+PARAM_CLOUD_TND = 3241
+PARAM_LW_TND = 3242
+PARAM_ALBEDO_FC = 3243
+PARAM_RGN_SRF_FC = 3244
+PARAM_RGN_NT_SRF_FC = 3245
+PARAM_CLOUD_LIQ = 3246
+PARAM_CLOUD_ICE = 3247
+PARAM_CLOUD_F = 3248
+PARAM_ICE_TND = 3249
+PARAM_ICE_AGE = 3250
+PARAM_TND_TEMP_AD = 3251
+PARAM_TND_HUM_AD = 3252
+PARAM_TND_ZWND_AD = 3253
+PARAM_TND_MWND_AD = 3254
 
 
 NAMES_UNITS = {
+  PARAM_UNKNOWN => ["unknown", "dat", nil, nil],
   PARAM_PRESSURE => ["Pressure", "PRES", "Pa" ,"air_pressure"],
   PARAM_PMSL => ["Pressure reduced to MSL", "PRMSL", "Pa" , "air_pressure_at_sea_level"],
   PARAM_PTND => ["Pressure tendency", "PTEND", "Pa/s", "tendency_of_air_pressure"],
@@ -418,7 +438,7 @@ NAMES_UNITS = {
   PARAM_GPT_HGT => ["Geopotential height", "HGTZ", "Gpm", "geopotential_height"],
   PARAM_GEOM_HGT => ["Geometric height", "DIST", "M", "altitude"],
   PARAM_HSTDV => ["Standard deviation of height", "HSTDV", "M", "equivalent_thickness_at_stp_of_atmosphere_o3_content"],
-#  PARAM_TOZNE => nil,
+  PARAM_TOZNE => ["Total ozone", "TOZNE", "Dobson", "equivalent_thickness_at_stp_of_atmosphere_o3_content"],
   PARAM_TEMP => ["Temperature", "TMP", "K", "air_temperature"],
   PARAM_VTEMP => ["Virtual temperature", "VTMP", "K", "virtual_temperature"],
   PARAM_POT_TEMP => ["Potential temperature", "POT", "K", "air_potential_temperature"],
@@ -429,16 +449,16 @@ NAMES_UNITS = {
   PARAM_DP_DEP => ["Dew point depression (or deficit)", "DEPR", "K", "dew_point_depression"],
   PARAM_LAPSE => ["Lapse rate", "LAPR", "K/m", "air_temperature_lapse_rate"],
   PARAM_VIS => ["Visibility", "VISIB", "M", "visibility_in_air"],
-  PARAM_RAD1 => ["Radar Spectra (1)", "RDSP1", "-", nil],
-  PARAM_RAD2 => ["Radar Spectra (2)", "RDSP2", "-", nil],
-  PARAM_RAD3 => ["Radar Spectra (3)", "RDSP3", "-", nil],
+  PARAM_RAD1 => ["Radar Spectra (1)", "RDSP1", nil, nil],
+  PARAM_RAD2 => ["Radar Spectra (2)", "RDSP2", nil, nil],
+  PARAM_RAD3 => ["Radar Spectra (3)", "RDSP3", nil, nil],
 #  PARAM_PLI => nil,
   PARAM_TANOM => ["Temperature anomaly", "TMPA", "K", "air_temperature_anomaly"],
   PARAM_PANOM => ["Pressure anomaly", "PRESA", "Pa", "air_pressure_anomaly"],
   PARAM_ZANOM => ["Geopotential height anomaly", "GPA", "Gpm", "geopotential_height_anomaly"],
-  PARAM_WAV1 => ["Wave Spectra (1)", "WVSP1", "-", nil],
-  PARAM_WAV2 => ["Wave Spectra (2)", "WVSP2", "-", nil],
-  PARAM_WAV3 => ["Wave Spectra (3)", "WVSP3", "-", nil],
+  PARAM_WAV1 => ["Wave Spectra (1)", "WVSP1", nil, nil],
+  PARAM_WAV2 => ["Wave Spectra (2)", "WVSP2", nil, nil],
+  PARAM_WAV3 => ["Wave Spectra (3)", "WVSP3", nil, nil],
   PARAM_WND_DIR => ["Wind direction", "WDIR", "Deg. true", "wind_from_direction"],
   PARAM_WND_SPEED => ["Wind speed", "WIND", "m/s", "wind_speed"],
   PARAM_U_WIND => ["u-component of wind", "UGRD", "m/s", "eastward_wind"],
@@ -564,6 +584,118 @@ NAMES_UNITS = {
   PARAM_PRESSURE_DEP => nil,
   PARAM_LAST_ENTRY => nil,
 
+
+## NCEP Edition 2
+#128	MSLSA	Pa		Mean Sea Level Pressure (Standard Atmosphere Reduction)		
+#129	MSLMA	Pa		Mean Sea Level Pressure (MAPS System Reduction)		
+#130	MSLET	Pa		Mean Sea Level Pressure (ETA Model Reduction)		
+  PARAM_LFT_X => ["Surface lifted index", "LFT_X", "K", nil],		
+  PARAM_4LFTX => ["Best (4 layer) lifted index", "4LFTX", "K", nil],		
+#133	K X	K		K index		
+#134	S X	K		Sweat index		
+#135	MCONV	kg/kg/s		Horizontal moisture divergence		
+  PARAM_VW_SH => ["Vertical speed shear", "VW_SH", "1/s", "wind_speed_shear"],
+#137	TSLSA	Pa/s		3-hr pressure tendency (Std. Atmos. Reduction)		
+#138	BVF 2	1/s2		Brunt-Vaisala frequency (squared)		
+#139	PV MW	1/s/m		Potential vorticity (density weighted)		
+#140	CRAIN	(yes=1; no=0)		Categorical rain		
+#141	CFRZR	(yes=1; no=0)		Categorical freezing rain		
+#142	CICEP	(yes=1; no=0)		Categorical ice pellets		
+#143	CSNOW	(yes=1; no=0)		Categorical snow		
+  PARAM_SOILW => ["Volumetric soil moisture content", "SOILW", "fraction", nil],
+#145	PEVPR	W/m**2		Potential evaporation rate		
+#146	CWORK	J/kg		Cloud workfunction		
+#147	U-GWD	N/m**2		Zonal flux of gravity wave stress		
+#148	V-GWD	N/m**2		Meridional flux of gravity wave stress		
+#149	PV	m**2/s/kg		Potential vorticity		
+#150	COVMZ	m2/s2		Covariance between meridional and zonal components of the wind. Defined as [uv]-[u][v], where "[]" indicates the mean over the indicated time span.		
+#151	COVTZ1	K*m/s		Covariance between temperature and zonal component of the wind. Defined as [uT]-[u][T], where "[]" indicates the mean over the indicated time span.		
+#152	COVTM	K*m/s		Covariance between temperature and meridional component of the wind. Defined as [vT]-[v][T], where "[]" indicates the mean over the indicated time span.		
+#153	CLWMR	Kg/kg		Cloud water		
+  PARAM_O3MR => ["Ozone mixing ratio", "O3MR", "Kg/kg", nil],
+#155	GFLUX	W/m2		Ground Heat Flux		
+  PARAM_CIN => ["Convective inhibition", "CIN", "J/kg", nil],
+  PARAM_CAPE => ["Convective Available Potential Energy", "CAPE", "J/kg", nil],		
+#158	TKE	J/kg		Turbulent Kinetic Energy		
+#159	CONDP	Pa		Condensation pressure of parcel lifted from indicated surface		
+#160	CSUSF	W/m2		Clear Sky Upward Solar Flux		
+#161	CSDSF	W/m2		Clear Sky Downward Solar Flux		
+#162	CSULF	W/m2		Clear Sky upward long wave flux		
+#163	CSDLF	W/m2		Clear Sky downward long wave flux		
+#164	CFNSF	W/m2		Cloud forcing net solar flux		
+#165	CFNLF	W/m2		Cloud forcing net long wave flux		
+#166	VBDSF	W/m2		Visible beam downward solar flux		
+#167	VDDSF	W/m2		Visible diffuse downward solar flux		
+#168	NBDSF	W/m2		Near IR beam downward solar flux		
+#169	NDDSF	W/m2		Near IR diffuse downward solar flux		
+#172	M FLX	N/m2		Momentum flux		
+#173	LMH	non-dim		Mass point model surface		
+#174	LMV	non-dim		Velocity point model surface		
+#175	MLYNO	non-dim		Model layer number (from bottom up)		
+#176	NLAT	deg		latitude (-90 to +90)		
+#177	ELON	deg		east longitude (0-360)		
+#181	LPS X	1/m		x-gradient of log pressure		
+#182	LPS Y	1/m		y-gradient of log pressure		
+#183	HGT X	m/m		x-gradient of height		
+#184	HGT Y	m/m		y-gradient of height		
+#189	VPTMP	K		Virtual potential temperature		
+#190	HLCY	m2/s2		Storm relative helicity		
+#191	PROB	numeric		Probability from ensemble		
+#192	PROBN	numeric		Probability from ensemble normalized with respect to climate expectancy		
+#193	POP	%		Probability of precipitation		
+#194	CPOFP	%		Probability of frozen precipitation		
+#195	CPOZP	%		Probability of freezing precipitation		
+#196	USTM	m/s		u-component of storm motion		
+#197	VSTM	m/s		v-component of storm motion		
+#201	ICWAT	%		Ice-free water surface		
+#204	DSWRF	W/m2		downward short wave rad. flux		
+#205	DLWRF	W/m2		downward long wave rad. flux		
+#206	UVI	J/m2		Ultra violet index (1 hour integration centered at solar noon)		
+#207	MSTAV	%		Moisture availability		
+#208	SFEXC	(kg/m3)(m/s)		Exchange coefficient		
+#209	MIXLY	integer		No. of mixed layers next to surface		
+#211	USWRF	W/m2		upward short wave rad. flux		
+#212	ULWRF	W/m2		upward long wave rad. flux		
+#213	CDLYR	%		Amount of non-convective cloud		
+#214	CPRAT	kg/m2/s		Convective Precipitation rate		
+#215	TTDIA	K/s		Temperature tendency by all physics		
+#216	TTRAD	K/s		Temperature tendency by all radiation		
+#217	TTPHY	K/s		Temperature tendency by non-radiation physics		
+#218	PREIX	fraction		precip.index(0.0-1.00)(see note)		
+#219	TSD1D	K		Std. dev. of IR T over 1x1 deg area		
+#220	NLGSP	ln(kPa)		Natural log of surface pressure		
+  PARAM_HPBL => ["Planetary boundary layer height", "HPBL", "m", nil],
+  PARAM_5WAVH => ["5-wave geopotential height", "5WAVH", "gpm", nil],
+#223	C WAT	kg/m2		Plant canopy surface water		
+#226	BMIXL	m		Blackadar's mixing length scale		
+#227	AMIXL	m		Asymptotic mixing length scale		
+#228	PEVAP	kg/m2		Potential evaporation		
+#229	SNOHF	W/m2		Snow phase-change heat flux
+  PARAM_5WAVA =>["5-wave geopotential height anomaly", "5WAVA", "gpm"],
+#231	MFLUX	Pa/s		Convective cloud mass flux		
+#232	DTRF	W/m2		Downward total radiation flux		
+#233	UTRF	W/m2		Upward total radiation flux		
+#234	BGRUN	kg/m2		Baseflow-groundwater runoff		
+#235	SSRUN	kg/m2		Storm surface runoff		
+#237	03TOT	Kg/m2		Total ozone		
+#238	SNO C	percent		Snow cover		
+#239	SNO T	K		Snow temperature		
+#241	LRGHR	K/s		Large scale condensat. heat rate		
+#242	CNVHR	K/s		Deep convective heating rate		
+#243	CNVMR	kg/kg/s		Deep convective moistening rate		
+#244	SHAHR	K/s		Shallow convective heating rate		
+#245	SHAMR	kg/kg/s		Shallow convective moistening rate		
+#246	VDFHR	K/s		Vertical diffusion heating rate		
+#247	VDFUA	m/s2		Vertical diffusion zonal acceleration		
+#248	VDFVA	m/s2		Vertical diffusion meridional accel		
+#249	VDFMR	kg/kg/s		Vertical diffusion moistening rate		
+#250	SWHR	K/s		Solar radiative heating rate		
+#251	LWHR	K/s		long wave radiative heating rate		
+#252	CD	non-dim		Drag coefficient		
+#253	FRICV	m/s		Friction velocity		
+#254	RI	non-dim.		Richardson number
+
+
 ## ECMWF 128  #incomplite
 #  4 => PARAM_EQPT_TEMP,
 #  5 => PARAM_SEPT_TEMP,
@@ -577,8 +709,8 @@ NAMES_UNITS = {
 #  26 => PARAM_LAKE_COV
   PARAM_LVEG_COV => ["Low vegetation cover", "CVL", "Fraction", nil],
   PARAM_HVEG_COV => ["High vegetation cover", "CVH", "Fraction", nil],
-  PARAM_TYP_LVEG => ["Type of low vegetation", "TVL", "-", nil],
-  PARAM_TYP_HVEG => ["Type of high vegetation", "TVH", "-", nil],
+  PARAM_TYP_LVEG => ["Type of low vegetation", "TVL", nil, nil],
+  PARAM_TYP_HVEG => ["Type of high vegetation", "TVH", nil, nil],
   PARAM_ICE_COV => ["Sea-ice cover", "CI", "Fraction", "sea_ice_area_fraction"],
   PARAM_SNOW_ALBD => ["Snow albedo", "ASN", "Fraction", nil],
   PARAM_SNOW_DENS => ["Snow density", "RSN", "kg/m3", nil],
@@ -607,7 +739,7 @@ NAMES_UNITS = {
 #  140 => PARAM_SOIL_WET,
 #  144 => PARAM_SNOW_FALL,
 
-  PARAM_CHNK => ["Charnock", "CHNK", "-", nil],
+  PARAM_CHNK => ["Charnock", "CHNK", nil, nil],
 #  149 => PARAM_RDN_SRF,
 #  150 => PARAM_RDN_TOP,
 
@@ -615,10 +747,10 @@ NAMES_UNITS = {
 #  154 => PARAM_HT_LW,
 
 #  159 => PARAM_BL_HGT,
-  PARAM_ORG_STDV => ["Standard deciation of orography", "SDOR", "-", nil],
-  PARAM_SGORG_ANIS => ["Anisotropy of sub-gridscale orography", "ISOR", "-", nil],
+  PARAM_ORG_STDV => ["Standard deciation of orography", "SDOR", nil, nil],
+  PARAM_SGORG_ANIS => ["Anisotropy of sub-gridscale orography", "ISOR", nil, nil],
   PARAM_SGORG_ANGL => ["Angle of sub-gridscale orography", "ANOR", "rad", nil],
-  PARAM_SGORG_SLOP => ["Slope of sub-gridscale orography", "SLOR", "-", nil],
+  PARAM_SGORG_SLOP => ["Slope of sub-gridscale orography", "SLOR", nil, nil],
 
 #  169 => PARAM_RND_SOL_SRFD,
 
@@ -675,7 +807,7 @@ PARAM_OZON_TOT => ["Total column ozone", "TCO3", "Dobson", nil],
 #  231 => PARAM_HT_SRF_IN,
 #  232 => PARAM_MST_IN,
 #  233 => PARAM_APP_HUM_SRF,
-  PARAM_LRGN_HT_SRF => ["Logarithm of sruface roughness length for heat", "LSRH", "-", nil],
+  PARAM_LRGN_HT_SRF => ["Logarithm of sruface roughness length for heat", "LSRH", nil, nil],
   PARAM_SKIN_TEMP => ["Skin temperature", "SKT", "K", nil],
 
   PARAM_SNOW_TEMP => ["Temperature of snow layer", "TSN", "K", nil],
@@ -701,6 +833,7 @@ nil => nil
 
 
 PARAMS_0 = {
+  0 => PARAM_UNKNOWN,
   1 => PARAM_PRESSURE,
   2 => PARAM_GPT_HGT,
   3 => PARAM_GEOM_HGT,
@@ -770,6 +903,7 @@ PARAMS_0 = {
 }
 
 PARAMS_2 = {
+  0 => PARAM_UNKNOWN,
   1 => PARAM_PRESSURE,
   2 => PARAM_PMSL,
   3 => PARAM_PTND,
@@ -779,7 +913,7 @@ PARAMS_2 = {
   7 => PARAM_GPT_HGT,
   8 => PARAM_GEOM_HGT,
   9 => PARAM_HSTDV,
-  #10 => PARAM_TOZNE,
+  10 => PARAM_TOZNE,
   11 => PARAM_TEMP,
   12 => PARAM_VTEMP,
   13 => PARAM_POT_TEMP,
@@ -899,8 +1033,22 @@ PARAMS_2 = {
   127 => PARAM_IMAGE
 }
 
+PARAMS_NCEP_2 = {   #incomplete
+  131 => PARAM_LFT_X,
+  132 => PARAM_4LFTX,
+  136 => PARAM_VW_SH,
+  144 => PARAM_SOILW,
+  154 => PARAM_O3MR,
+  156 => PARAM_CIN,
+  157 => PARAM_CAPE,
+  221 => PARAM_HPBL,
+  222 => PARAM_5WAVH,
+  230 => PARAM_5WAVA,
+}
+
 
 PARAMS_ECMWF_128 = {   #inconmplte
+  0 => PARAM_UNKNOWN,
   1 => PARAM_STRM_FUNC,
   2 => PARAM_VPOT,
   3 => PARAM_POT_TEMP,
