@@ -218,6 +218,22 @@ NOMENCLATURE
     * an object representing the file in which data are stored. Its class
       depends on the file type. nil is returned if the data is not in a file.
 
+
+---transpose(*dims)
+    Transpose. Argument specification is as in NArray.
+
+---axis_draw_positive
+    Returns the direction to plot the axis (meaningful only if self is a
+    coordinate variable.) 
+    
+    The current implementation is based on NetCDF conventions, 
+    so REDEFINE IT IN SUBCLASSES if it is not appropriate.
+    
+    Return values
+    * true: axis should be drawn in the increasing order (to right/upward)
+    * false: axis should be drawn in the decreasing order
+    * nil: not specified
+
 ---coerce(other)
     For Numeric operators. (If you do not know it, see a manual or book of Ruby)
 
@@ -301,11 +317,28 @@ These methods returns a NArray (not a VArray).
       end
 
       def val
-	 if @mapping
-	    @varray.ary[*@mapping.slicer]  # 1-step mapping is assumed
-	 else
-	    @ary.dup
-	 end
+	if @mapping
+	  ary = @varray.ary
+	  slicer = @mapping.slicer
+	  if ary.is_a?(NArray) || ary.is_a?(NArrayMiss)
+	    # interpret Hash slicers for NArray/NArrayMiss
+	    slicer.collect!{ |sl|
+	      if sl.is_a?(Hash)
+		range, step = sl.to_a[0]
+		first = range.first
+		last = range.last
+		step = -step if ( (last-first)*step < 0 )
+		length = (last-first) / step + 1
+		first + step * NArray.int(length).indgen!
+	      else
+		sl
+	      end
+	    }
+	  end
+	  ary[*slicer]
+	else
+	  @ary.dup
+	end
       end
 
       def val=(narray)
@@ -477,6 +510,14 @@ These methods returns a NArray (not a VArray).
 	end
       end
 
+      def long_name
+	attr['long_name']
+      end
+
+      def long_name=(nm)
+	attr['long_name'] = nm
+      end
+
       def [](*slicer)
 	 slicer = __rubber_expansion( slicer )
 	 mapping = SubsetMapping.new(self.shape_current, slicer)
@@ -513,6 +554,18 @@ These methods returns a NArray (not a VArray).
 	VArray.new( val.transpose(*dims), attr_copy, name )
       end
 
+      def axis_draw_positive
+	positive = attr['positive']
+	case positive
+	when 'up'
+	  true
+	when 'down'
+	  false
+	else
+	  nil    # not defined, or if not 'up' or 'down'
+	end  
+      end
+
       ### < NArray methods > ###
 
       ## ToDo: implement units handling
@@ -840,9 +893,11 @@ end
 if $0 == __FILE__
    include NumRu
    p va = VArray.new( NArray.int(6,2,3).indgen!, nil, 'va' )
-   va.set_att("units","m")
+   va.units="m"
+   va.long_name="test data"
    vs = va[2..4,0,0..1]
    p "@@@",vs.rank,vs.shape,vs.total,vs.val,vs.get_att("name")
+   p '@@@@',vs.long_name,vs.units.to_s
    vs.val=999
    p "*1*",va
    co,=va.coerce(UNumeric.new(1,Units.new("rad")))
