@@ -165,12 +165,22 @@ module NumRu
     alias total length
 
     def val
-      val = NArray.new(self.ntype,*@shape)
+      val = nil
       loop_multi_dim_index( @varrays.shape ){|index|
 	vidx = Array.new
 	for d in 0...@rank
 	  vidx[d] = (@bound_idx[d][index[d]])..(@bound_idx[d][index[d]+1]-1)
 	end
+	v = @varrays[*index].val
+	if !val
+	  if NArray===v
+	    val = NArray.new(self.ntype,*@shape)
+	  elsif NArrayMiss===v
+	    val = NArrayMiss.new(self.ntype,*@shape)
+	  else
+	    raise TypeError, "Unexpected type #{v.class}"
+	  end
+	end
 	val[*vidx] = @varrays[*index].val
       }
       val
@@ -214,6 +224,18 @@ module NumRu
       val
     end
 
+    def ntype
+      __ntype(@first_vary)
+    end
+
+    def typecode
+      @first_vary.typecode
+    end
+
+    def file
+      @varrays.collect{|va| va.file}
+    end
+
     def name=(nm)
       raise ArgumentError, "name should be a String" if ! nm.is_a?(String)
       @name = nm
@@ -237,6 +259,11 @@ module NumRu
     #end
     undef reshape!
 
+    ## Not needed, so far:
+    #def convention
+    #  @first_vary.convention
+    #end
+
     ################# private #####################
     private
 
@@ -365,7 +392,7 @@ if __FILE__ == $0
   varrays = NArray[ [ va00.copy, nil       ],
                     [ va01.copy, va11.copy ] ]
   p vac2 = VArrayComposite.new(varrays)
-  p vac.ntype
+  p vac.ntype, vac.typecode
   p vac.val
   vacs = vac[1..3,0..3,true]
   p vacs.val
